package com.cardiff.client_project.mapper;

import com.cardiff.client_project.pojo.dto.HospitalDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.stream.Collectors;

@Repository
public class HospitalMapper {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public List<HospitalDTO> findAvailableBeds(String name, String location, Integer minBeds) {
        String sql = """
            SELECT 
                h.id AS hospital_id,
                h.name AS hospital_name,
                h.address AS location,
                h.phone,
                h.maxAmount AS total_beds,
                (h.maxAmount - h.amountPatient) AS available_beds,
                ROUND(((h.amountPatient * 1.0) / h.maxAmount * 100), 1) AS occupancy_rate
            FROM 
                hospital h
            WHERE 
                h.status = 1 
                AND (h.maxAmount - h.amountPatient) > ?
                AND (? IS NULL OR h.name LIKE ?)
                AND (? IS NULL OR h.address LIKE ?)
            ORDER BY 
                available_beds DESC,
                occupancy_rate ASC
        """;

        return jdbcTemplate.query(sql, 
            ps -> {
                int paramIndex = 1;
                ps.setInt(paramIndex++, minBeds != null ? minBeds : 0);
                ps.setString(paramIndex++, name);
                ps.setString(paramIndex++, name != null ? "%" + name + "%" : "%");
                ps.setString(paramIndex++, location);
                ps.setString(paramIndex++, location != null ? "%" + location + "%" : "%");
            },
            (rs, rowNum) -> {
                HospitalDTO hospital = new HospitalDTO();
                hospital.setId(rs.getInt("hospital_id"));
                hospital.setName(rs.getString("hospital_name"));
                hospital.setLocation(rs.getString("location"));
                hospital.setPhone(rs.getString("phone"));
                hospital.setTotalBeds(rs.getInt("total_beds"));
                hospital.setAvailableBeds(rs.getInt("available_beds"));
                hospital.setOccupancyRate(rs.getDouble("occupancy_rate"));
                return hospital;
            });
    }

    public int insertHospital(HospitalDTO hospitalDTO) {
        String sql = """
            INSERT INTO hospital (name, address, phone, maxAmount, amountPatient, status)
            VALUES (?, ?, ?, ?, 0, 1)
        """;
        
        return jdbcTemplate.update(sql,
            hospitalDTO.getName(),
            hospitalDTO.getLocation(),
            hospitalDTO.getPhone(),
            hospitalDTO.getTotalBeds()
        );
    }

    public int updateHospital(HospitalDTO hospitalDTO) {
        String sql = """
            UPDATE hospital 
            SET name = ?, 
                address = ?, 
                phone = ?, 
                maxAmount = ?
            WHERE id = ?
        """;
        
        return jdbcTemplate.update(sql,
            hospitalDTO.getName(),
            hospitalDTO.getLocation(),
            hospitalDTO.getPhone(),
            hospitalDTO.getTotalBeds(),
            hospitalDTO.getId()
        );
    }

    public int[] deleteHospitalByIds(List<Integer> ids) {
        String sql = "DELETE FROM hospital WHERE id = ?";
        List<Object[]> batchArgs = ids.stream()
            .map(id -> new Object[]{id})
            .collect(Collectors.toList());
            
        return jdbcTemplate.batchUpdate(sql, batchArgs);
    }

    public List<HospitalDTO> findAllHospitals() {
        String sql = """
            SELECT 
                h.id AS hospital_id,
                h.name AS hospital_name,
                h.address AS location,
                h.phone,
                h.maxAmount AS total_beds,
                (h.maxAmount - h.amountPatient) AS available_beds,
                ROUND(((h.amountPatient * 1.0) / h.maxAmount * 100), 1) AS occupancy_rate
            FROM hospital h
            WHERE h.status = 1
            ORDER BY h.name
        """;
        
        return jdbcTemplate.query(sql, (rs, rowNum) -> {
            HospitalDTO hospital = new HospitalDTO();
            hospital.setId(rs.getInt("hospital_id"));
            hospital.setName(rs.getString("hospital_name"));
            hospital.setLocation(rs.getString("location"));
            hospital.setPhone(rs.getString("phone"));
            hospital.setTotalBeds(rs.getInt("total_beds"));
            hospital.setAvailableBeds(rs.getInt("available_beds"));
            hospital.setOccupancyRate(rs.getDouble("occupancy_rate"));
            return hospital;
        });
    }

    public int updateBedCount(int hospitalId, int currentPatients) {
        String sql = """
            UPDATE hospital 
            SET amountPatient = ? 
            WHERE id = ? 
            AND maxAmount >= ?
        """;
        
        return jdbcTemplate.update(sql, 
            currentPatients, 
            hospitalId, 
            currentPatients);
    }
}